import shutil
from glob import glob
from pathlib import Path

from scriptengine.exceptions import ScriptEngineTaskRunError
from scriptengine.tasks.core import Task, timed_runner


class Copy(Task):
    """SE base.copy task, copies files and directories:

        base.copy:
            src: <source>
            dst: <destination>
            ignore_not_found: <true_or_false>  # optional

    where <source> is a file/directory name or a glob (wildcard) pattern, or a
    list with items thereof. <destination> is a file or directory name. If
    <ignore_not_found> is false (default), a ScriptEngineTaskRunError is raised
    if no source files are found. If true, only a warning is written and SE
    continues.
    """

    _required_arguments = (
        "src",
        "dst",
    )

    def __init__(self, arguments):
        Copy.check_arguments(arguments)
        super().__init__(arguments)

    @timed_runner
    def run(self, context):

        src_arg = self.getarg("src", context)
        dst_arg = self.getarg("dst", context)
        self.log_info(f"Copy: {src_arg} --> {dst_arg}")

        sources = []
        for s in src_arg if isinstance(src_arg, list) else [src_arg]:
            try:
                sources.extend(glob(s))
            except TypeError as e:
                self.log_error(f"Argument error in 'src': {e}")
                raise ScriptEngineTaskRunError
        self.log_debug(f"Copy sources: {sources}")

        if not sources:
            if self.getarg("ignore_not_found", context, default=False):
                self.log_warning("No files found to copy")
                return
            else:
                self.log_error("No files found to copy")
                raise ScriptEngineTaskRunError

        try:
            dst = Path(dst_arg)
        except TypeError as e:
            self.log_error(f"Argument error in 'dst': {e}")
            raise ScriptEngineTaskRunError
        self.log_debug(f"Copy destination: {dst}")

        if dst.is_file():
            if len(sources) > 1:
                self.log_error("Cannot copy multiple files onto one file")
                raise ScriptEngineTaskRunError
            if Path(sources[0]).is_dir():
                self.log_error("Cannot copy a directory onto a file")
                raise ScriptEngineTaskRunError
            self.log_warning(f"Destination file exists, overwriting '{dst}'")

        for src in (Path(s) for s in sources):
            if src.is_file():
                self.log_debug(f"Copy file '{src} --> {dst}")
                shutil.copy(src, dst)
            elif src.is_dir():
                self.log_debug(f"Copy directory '{src} --> {dst}")
                shutil.copytree(src, dst, symlinks=True)
            else:
                self.log_error(f"'src' is not a valid file or directory: '{src}'")
                raise ScriptEngineTaskRunError



from pathlib import Path

from scriptengine.exceptions import ScriptEngineTaskRunError
from scriptengine.tasks.core import Task, timed_runner


class MakeDir(Task):
    """SE base.make_dir task, creates directories:

        base.make_dir:
            path: <dir_path>

    where <dir_path> is the name of the new directory or list of such.
    If a directory exists already, a warning is written and SE continues. If a
    file with the same name exists, a ScriptEngineTaskRunError is raised.
    """

    _required_arguments = ("path",)

    def __init__(self, arguments):
        MakeDir.check_arguments(arguments)
        super().__init__(arguments)

    @timed_runner
    def run(self, context):

        path_arg = self.getarg("path", context)
        self.log_info(f"MakeDir: {path_arg}")

        for path in (
            Path(p) for p in (path_arg if isinstance(path_arg, list) else [path_arg])
        ):
            self.log_debug(f"Create directory '{path}'")
            try:
                path.mkdir(parents=True)
            except FileExistsError as e:
                if path.is_dir():
                    self.log_warning(f"Error creating '{path}': {e}")
                else:
                    self.log_error(
                        f"File '{path}' exists already, can not create directory"
                    )
                    raise ScriptEngineTaskRunError



from glob import glob
from pathlib import Path

from scriptengine.exceptions import (
    ScriptEngineTaskArgumentMissingError,
    ScriptEngineTaskRunError,
)
from scriptengine.tasks.core import Task, timed_runner


class Link(Task):
    """SE base.link task, creates symlinks:

        base.link:
            link: <link>  # optional, defaults to '.'
            target: <target>

    where <link> is a file or directory name and <target> is a file/directory
    name, a glob pattern, or a list thereof. If it is a glob pattern or a list,
    <link> must be a directory.  A symlink with the specified name is created
    that points to the target. If the link name is an existing directory, the
    symlink named after the target basename is created inside this directory.
    When multiple targets are specified (list or glob pattern) the links are
    created inside the <link> directory for each target.
    """

    #   _required_arguments = ("target",)

    def __init__(self, arguments):
        Link.check_arguments(arguments)
        super().__init__(arguments)

    @timed_runner
    def run(self, context):

        target_arg = self.getarg("src", context, default=False)
        if target_arg:
            self.log_warning("The 'src' argument is deprecated, use 'target' instead!")
        else:
            target_arg = self.getarg("target", context, default=False)
            if not target_arg:
                self.log_error("Missing 'target' argument")
                raise ScriptEngineTaskArgumentMissingError

        link_arg = self.getarg("dst", context, default=False)
        if link_arg:
            self.log_warning("The 'dst' argument is deprecated, use 'link' instead!")
        else:
            link_arg = self.getarg("link", context, default=".")

        self.log_info(f"Link: '{link_arg}' --> '{target_arg}'")

        target_list = []
        for t in target_arg if isinstance(target_arg, list) else [target_arg]:
            try:
                target_list.extend(glob(t))
            except TypeError as e:
                self.log_error(f"Argument error in 'target': {e}")
                raise ScriptEngineTaskRunError
        self.log_debug(f"Link targets: {target_list}")

        if not target_list:
            self.log_warning("No valid targets, therefore no links created")
            return

        try:
            link = Path(link_arg)
        except TypeError as e:
            self.log_error(f"Argument error in 'link': {e}")
            raise ScriptEngineTaskRunError

        if len(target_list) > 1 and not link.is_dir():
            self.log_error("If multiple targets are given, link must be a directory")
            raise ScriptEngineTaskRunError

        for target in (Path(t) for t in target_list):
            if link.is_dir():
                (link / target.name).symlink_to(target)
            else:
                link.symlink_to(target)



import shutil
from glob import glob
from pathlib import Path

from scriptengine.exceptions import ScriptEngineTaskRunError
from scriptengine.tasks.core import Task, timed_runner


class Remove(Task):
    """SE base.remove task, deletes files or directories:

        base.remove:
            path: <path>
            ignore_not_found: <true_or_false>  # optional

    where <path> is a file/directory name or a glob (wildcard) pattern, or a
    list with items thereof.
    If <ignore_not_found> is false (default), a ScriptEngineTaskRunError is
    raised if a file or directory from <path> is found. If true, only a warning
    is written and SE continues.
    """

    _required_arguments = ("path",)

    def __init__(self, arguments):
        Remove.check_arguments(arguments)
        super().__init__(arguments)

    @timed_runner
    def run(self, context):

        path_arg = self.getarg("path", context)
        self.log_info(f"Remove '{path_arg}'")

        path_list = []
        for p in path_arg if isinstance(path_arg, list) else [path_arg]:
            try:
                path_list.extend(glob(p))
            except TypeError as e:
                self.log_error(f"Argument error in 'path': {e}")
                raise ScriptEngineTaskRunError
        self.log_debug(f"Remove files/dirs: {path_list}")

        if not path_list:
            if self.getarg("ignore_not_found", context, default=False):
                self.log_warning("Non-existing path, nothing to remove")
                return
            else:
                self.log_warning("Non-existing path")
                self.log_warning(
                    "Deprecation warning! base.remove has not found anything to "
                    "delete but ignore_not_found was not true. In the future, "
                    "this will lead to an error!"
                )
                return
            #   self.log_error("Non-existing path")
            #   raise ScriptEngineTaskRunError

        for path in (Path(p) for p in path_list):
            if path.is_dir():
                self.log_debug(f"Remove directory: '{path}'")
                try:
                    shutil.rmtree(path)
                except PermissionError as e:
                    self.log_error(f"Cannot remove directory: {e}")
                    raise ScriptEngineTaskRunError
            else:
                self.log_debug(f"Remove file: '{path}'")
                try:
                    path.unlink()
                except PermissionError as e:
                    self.log_error(f"Cannot remove file: {e}")
                    raise ScriptEngineTaskRunError
